#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib
import matplotlib.pyplot as plt

import os
import sys
import copy
import time

import math
import numpy as np
import threading as thread

from Explorer import Explorer

from gui_utils import *

if __name__ == "__main__":
    color = "blue"
    explorer = Explorer() ## instantiate the explorer agent
    explorer.start() ## start the agent
    
    ## Main loop continuously plot the map, targets and plan (once per second)
    fig, (ax1) = plt.subplots(nrows = 1, ncols = 1)
    plt.ion()

    while not explorer.is_stopped():
        ax1.clear()
        print('[Gui] Redraw state')
        explorer.mutex_gridmap.acquire()
        if explorer.gridmap is not None : explorer.gridmap.plot(ax1)
        explorer.mutex_gridmap.release()

        for ax in [ax1]:
            explorer.mutex_path_following.acquire()
            if explorer.path is not None:
                draw_path(explorer.path, ax, 'orange')
                goal = explorer.robot.navigation_goal 
                if goal is not None: ax.plot(goal.position.x, goal.position.y, color = 'blue', marker = "o", linestyle = "None")
            explorer.mutex_path_following.release()

            explorer.mutex_planning.acquire()
            if explorer.frontiers is not None and len(explorer.frontiers):
                cordx = [p.position.x for p in explorer.frontiers]; cordx.append(cordx[0])
                cordy = [p.position.y for p in explorer.frontiers]; cordy.append(cordy[0])
                ax.plot(cordx, cordy, color = 'blue', marker = ".", linestyle = "None")
            if explorer.planned_path is not None: draw_path(explorer.planned_path, ax, 'blue')
            if explorer.simplified_planned_path is not None: draw_path(explorer.simplified_planned_path, ax, 'red')
            explorer.mutex_planning.release()
            ax.set_xlabel('x[m]')
            ax.set_aspect('equal', 'box')
        ax1.set_ylabel('y[m]')
        plt.show()
        print("[Gui] drawing done")
        plt.pause(2) ## pause to throttle the plotting
    explorer.__del__()

