import numpy as np

import matplotlib.pyplot as plt
from matplotlib.patches import Circle, PathPatch

from Point2D import Point2D

def plot_path(ax, path, clr="r"):
    """Draw the path"""
    length = 0
    if path is not None:
        coords = np.asarray([(coords.x, coords.y) for coords in path.coords])
        ax.plot(coords[:,0], coords[:,1], '.', color=clr)
        ax.plot(coords[:,0], coords[:,1], '-', color=clr)

def plot_goals(ax, goals, neighborhood_size):
    """Draw the goal positions with neighborhood"""
    for goal in goals:
        x = goal.x
        y = goal.y
        #plot the point
        ax.plot([x], [y], ".k", markersize=7)
        #plot the neighborhood
        if(neighborhood_size > 0):
            circle = Circle((x,y), neighborhood_size, facecolor="yellow",edgecolor="orange", linewidth=1, alpha=0.2)
            ax.add_patch(circle)

def load_problem(problem_file):
    """Load problem from file"""
    goals = []
    data = np.genfromtxt(problem_file)
    for pt in data:
        coords = Point2D(pt[0], pt[1])
        goals.append(coords)
    return goals

def draw_tspn_solution(goals, radius, path, title):
    fig = plt.figure()
    ax = plt.axes()
    plot_goals(ax, goals, radius)
    plot_path(ax, path, 'r')
    plt.xlabel('x[m]')
    plt.ylabel('y[m]')
    plt.axis('equal')
    plt.title(title)
    plt.pause(1)
    return fig

def save_image(fig, msg, image_file):
    fig.savefig(image_file, bbox_inches='tight')
    print(msg, image_file) 
    return

